/* eagle2.c in ~/eagle */
#include "eagle.h"

void set_defaults(w)
          Window w;
{
           Window root; int x, y;
           unsigned int x_width, y_height, border, depth;
/*(1)*/    w_present_eagle = w;
/*(2)*/    gc_eagle = XCreateGC(display_eagle, w, 0, 0);
/*(3)*/    set_foreground(BlackPixel(display_eagle,0));
/*(3)*/    set_background(WhitePixel(display_eagle,0));
/*(4)*/    set_stipple_patterns();
/*(5)*/    line_width_eagle = 1; line_style_eagle = LineSolid;
/*(5)*/    XSetLineAttributes(display_eagle, gc_eagle, line_width_eagle,
		              line_style_eagle, CapButt, JoinMiter );
/*(6)*/    XGetGeometry(display_eagle, w, &root, &x, &y, &x_width, &y_height,
                        &border, &depth);
/*(6)*/    m_trans_def(-1., -1., 1., 1., 0, 0, x_width, y_height);
/*(7)*/    set_font(1);
/*(8)*/    p_control_eagle = 0;
}

unsigned long set_color(red, green, blue)
    int red, green, blue;
{
    Status result; Colormap cmap; XColor c1;
    cmap = DefaultColormap(display_eagle, 0);
    if (red < 0)    red = 0;     if (red > 255)   red = 255;
    if (green < 0)  green = 0;   if (green > 255) green = 255;
    if (blue < 0)   blue = 0;    if (blue > 255)  blue = 255;
    c1.red   = (long) (red*65535L/255);
    c1.green = (long) (green*65535L/255);
    c1.blue  = (long) (blue*65535L/255);
    result = XAllocColor(display_eagle, cmap, &c1);
    if (result == 0)
    { printf("There is no color cell\n"); return BlackPixel(display_eagle,0); }
    return c1.pixel;
}

void set_foreground(cname)
    unsigned long cname;
{
    foreground_eagle = cname;
    XSetForeground(display_eagle, gc_eagle, foreground_eagle);
}

void set_background(cname)
    unsigned long cname;
{
    background_eagle = cname;
    XSetBackground(display_eagle, gc_eagle, background_eagle);
    XSetWindowBorder(display_eagle, w_present_eagle, background_eagle);
    XSetWindowBackground(display_eagle, w_present_eagle, background_eagle);
}

void set_fill_style(fp) 
    int fp;
{
    int fnum, fcen;
    fnum = 9; fcen = 5;
    if (fp >= 10) {
       if (fp > 10+fnum) fp = 10+fcen;
       XSetFillStyle(display_eagle, gc_eagle, FillStippled);
       XSetStipple(display_eagle, gc_eagle, bitmap_eagle[fp-10]);
    }
    else XSetFillStyle(display_eagle, gc_eagle, FillSolid);
}

void set_stipple_patterns()
{
    char data0[3]; int i, j;
    static char data[10][3] = 
    {{0x00,0x00,0x00}, {0x00,0x02,0x00}, {0x04,0x00,0x01},
     {0x04,0x01,0x04}, {0x05,0x00,0x05}, {0x05,0x02,0x05},
     {0x03,0x06,0x03}, {0x03,0x07,0x06}, {0x07,0x05,0x07},
     {0x07,0x07,0x07}};

    for (i=0; i<10; i++) {
      for (j=0; j<3; j++) data0[j] = data[i][j];
      bitmap_eagle[i] = XCreateBitmapFromData(display_eagle, w_present_eagle,
		        data0, 3, 3);
    }
}

void set_line_width(k)
    unsigned int k;
{
    line_width_eagle = k;
    XSetLineAttributes(display_eagle, gc_eagle, line_width_eagle,
		       line_style_eagle, CapButt, JoinMiter);
}

void set_line_style(k)
    int k;  /* k = LineSolid, LineOnOffDash or LineDoubleDash */
{
    line_style_eagle = k;
    XSetLineAttributes(display_eagle, gc_eagle, line_width_eagle,
		       line_style_eagle, CapButt, JoinMiter);
}

void set_font(font_id)
    int font_id;
{
    Font f1;
    if (font_id <= 0) {
      f1 = XLoadFont(display_eagle, "12x24");
      XSetFont(display_eagle, gc_eagle, f1); font_id_eagle = 0;
    }
    if (font_id == 1) {
      f1 = XLoadFont(display_eagle, "10x20");
      XSetFont(display_eagle, gc_eagle, f1); font_id_eagle = 1;
    }
    if (font_id == 2) {
      f1 = XLoadFont(display_eagle, "8x16");
      XSetFont(display_eagle, gc_eagle, f1); font_id_eagle = 2;
    }
    if (font_id == 3) {
      f1 = XLoadFont(display_eagle, "7x14");
      XSetFont(display_eagle, gc_eagle, f1); font_id_eagle = 3;
    }
    if (font_id >= 4) {
      f1 = XLoadFont(display_eagle, "6x13");
      XSetFont(display_eagle, gc_eagle, f1); font_id_eagle = 4;
    }
}

void set_cursor(w)
    Window w;
{
    Cursor curs; Pixmap psource, pmask; Colormap cm;
    XColor cwhite, cblack, cc;
    static char source_data[32] =
    {0x0f, 0xf0, 0x11, 0x88, 0x21, 0x84, 0x41, 0x82,
     0x82, 0x41, 0x04, 0x20, 0x08, 0x10, 0x10, 0x08,
     0x10, 0x08, 0x08, 0x10, 0x04, 0x20, 0x82, 0x41,
     0x41, 0x82, 0x21, 0x84, 0x11, 0x88, 0x0f, 0xf0};
    static char mask_data[32] =
    {0x0f, 0xf0, 0x1f, 0xf8, 0x3f, 0xfc, 0x7f, 0xfe,
     0xfe, 0x7f, 0xfc, 0x3f, 0xf8, 0x1f, 0xf0, 0x0f,
     0xf0, 0x0f, 0xf8, 0x1f, 0xfc, 0x3f, 0xfe, 0x7f,
     0x7f, 0xfe, 0x3f, 0xfc, 0x1f, 0xf8, 0x0f, 0xf0};

    psource = XCreateBitmapFromData(display_eagle, w, source_data, 16, 16);
    pmask   = XCreateBitmapFromData(display_eagle, w, mask_data, 16, 16);
    cm = DefaultColormap(display_eagle, 0);
    XAllocNamedColor(display_eagle, cm, "blue", &cwhite, &cc);
    XAllocNamedColor(display_eagle, cm, "yellow", &cblack, &cc);
    curs = XCreatePixmapCursor(display_eagle, psource, pmask,
			       &cwhite, &cblack, 8, 8);
    XDefineCursor(display_eagle, w, curs);
}

void set_draw_mode(mode)
    int mode;
{
    unsigned long mask;
    if (mode == 1) {
      p_control_eagle = 0;
      mask = BlackPixel(display_eagle, 0) ^ WhitePixel(display_eagle, 0);
      XSetForeground(display_eagle, gc_eagle, 0xffffffff);
      XSetFunction(display_eagle, gc_eagle, GXxor);
      XSetPlaneMask(display_eagle, gc_eagle, mask);
    }
    else {
      mask = 0xffffffff;
      XSetForeground(display_eagle, gc_eagle, foreground_eagle);
      XSetBackground(display_eagle, gc_eagle, background_eagle);
      XSetFunction(display_eagle, gc_eagle, GXcopy);
      XSetPlaneMask(display_eagle, gc_eagle, mask);
    }
}
