/* eagle4.c in ~/eagle */
#include "eagle.h"

void m_trans_def(mx0, my0, mx1, my1, x0, y0, x_width, y_height)
    X_real mx0, my0, mx1, my1; int x0, y0, x_width, y_height;
{
    x_width = abs(x_width); y_height = abs(y_height);
    Ax_eagle = x_width/(mx1-mx0);  Bx_eagle = x0-Ax_eagle*mx0;
    Ay_eagle = y_height/(my0-my1); By_eagle = y0-Ay_eagle*my1;
}

void m_trans_m_to_x(mx, my, x, y)
    X_real mx, my; int *x, *y;
{
    *x = Ax_eagle*mx + Bx_eagle; *y = Ay_eagle*my + By_eagle;
}

void m_trans_x_to_m(x, y, mx, my)
    int x, y; X_real *mx, *my;
{
    *mx = (x-Bx_eagle)/Ax_eagle; *my = (y-By_eagle)/Ay_eagle;
}

void m_point(mx, my)
    X_real mx, my;
{
    int x, y;
    m_trans_m_to_x(mx, my, &x, &y); x_point(x, y);
}

void m_line(mx0, my0, mx1, my1)
    X_real mx0, my0, mx1, my1;
{
    int x0, y0, x1, y1;
    m_trans_m_to_x(mx0, my0, &x0, &y0); m_trans_m_to_x(mx1, my1, &x1, &y1);
    x_line(x0, y0, x1, y1);
}

void m_box(mx0, my0, mx1, my1, fp)
    X_real mx0, my0, mx1, my1; int fp;
{
    int x0, y0, x1, y1, x_width, y_height;
    m_trans_m_to_x(mx0, my0, &x0, &y0); m_trans_m_to_x(mx1, my1, &x1, &y1);
    x_width = Ax_eagle*(mx1-mx0); y_height = Ay_eagle*(my1-my0);
    if (x0 > x1) x0 = x1; if (y1 > y0) y1 = y0;
    x_box(x0, y1, x_width, y_height, fp);
}

void m_circle(mx, my, ma, mb, fp)
    X_real mx, my, ma, mb; int fp;
{
    int x, y, a, b;
    m_trans_m_to_x(mx, my, &x, &y); a = Ax_eagle*ma; b = Ay_eagle*mb;
    x_circle(x, y, a, b, fp);
}

void m_lines(n, mx, my)
    int n; X_real mx[], my[];
{
    int *x, *y; int i;
    x = (int *)malloc(sizeof(*x)*n); y = (int *)malloc(sizeof(*y)*n);
    for (i=0; i<n; i++) m_trans_m_to_x(mx[i], my[i], &x[i], &y[i]);
    x_lines(n, x, y); free(x); free(y);
}

void m_graph(n, mx0, mx1, my)
    int n; X_real mx0, mx1; X_real my[];
{
    int *y; int x0, x1, i;
    y = (int *)malloc(sizeof(*y)*(n+1));
    for (i=0; i<=n; i++) y[i] = Ay_eagle*my[i]+By_eagle;
    x0 = Ax_eagle*mx0+Bx_eagle; x1 = Ax_eagle*mx1+Bx_eagle;
    x_graph(n, x0, x1, y); free(y);
}

void m_polygon(n, mx, my, fp)
    int n, fp; X_real mx[], my[];
{
    int *x, *y; int i;
    x = (int *)malloc(sizeof(*x)*n); y = (int *)malloc(sizeof(*y)*n);
    for (i=0; i<n; i++) m_trans_m_to_x(mx[i], my[i], &x[i], &y[i]);
    x_polygon(n, x, y, fp); free(x); free(y);
}

int m_printf(mx, my, va_alist)
    X_real mx, my; va_dcl
{
    va_list args; int x, y, length; char string[32768]; char *format;
    va_start(args);
    format = va_arg(args, char *);
    vsprintf(string, format, args);
    length = strlen(string);
    m_trans_m_to_x(mx, my, &x, &y);
    XDrawString(display_eagle, w_present_eagle, gc_eagle, x, y, string, length);
    XFlush(display_eagle);
    va_end(args);
    if( (w_present_eagle == w_first_eagle) && (p_control_eagle == 1) ) {
      fprintf(file_id_eagle, "x_printf\n");
      fprintf(file_id_eagle, "%d %d %d\n", x, y, font_id_eagle);
      fprintf(file_id_eagle, string); fprintf(file_id_eagle, "\n");
    }
}

void m_hidden(z_min, z_max, x_ratio, y_ratio, x0, y0, x_width, y_width,
              array, Nx, Ny, x_direction, y_direction, x_intv, y_intv, fp)
     X_real z_min, z_max, x_ratio, y_ratio; X_real array[];
     int x0, y0, x_width, y_width, Nx, Ny, x_direction, y_direction;
     int x_intv, y_intv, fp;
{
     int x_total, x_diff, yz_total, z_height, CDy, base; X_real Az;
     int i, j, ij, inw, jnw, p_control, fpp; int xx[4], yy[4];
     unsigned long black, blue, forecolor, backcolor;
     int *y1, *y2;

     y1 = (int *)malloc(sizeof(*y1)*Nx); y2 = (int *)malloc(sizeof(*y2)*Nx);
     p_control = p_control_eagle; p_control_eagle = 0;
     forecolor = foreground_eagle; backcolor = background_eagle;
     black = set_color(0, 0, 0); blue = set_color(0, 0, 255);
     fpp = fp; if (fpp < 1) fpp = 10; if (fpp > 19) fpp = 15;

     x_total = x_width/x_ratio; x_diff = x_total - x_width;
     yz_total = y_width/y_ratio; z_height = yz_total - y_width;
     Az = z_height/(z_max-z_min); CDy = y0 + z_height;

     set_foreground(black);
     x_line(x0, y0+yz_total, x0+x_width, y0+yz_total);
     x_line(x0+x_width, y0+yz_total, x0+x_total, y0+z_height);
     x_line(x0+x_total, y0+z_height, x0+x_diff, y0+z_height);
     x_line(x0+x_diff, y0+z_height, x0, y0+yz_total);
     x_line(x0, y0+yz_total, x0, y0+y_width);
     x_line(x0+x_total, y0+z_height, x0+x_total, y0);
     x_line(x0+x_diff, y0+z_height, x0+x_diff, y0);

     if( (w_present_eagle == w_first_eagle) && (p_control == 1) ) {
       fprintf(file_id_eagle, "x_hidden\n");
       fprintf(file_id_eagle, "%lf %lf %lf %lf %d %d %d %d %d %d %d %d %d\n",
       z_min, z_max, x_ratio, y_ratio, x0, y0, x_width, y_width, Nx, Ny,
       x_intv, y_intv, fpp);
     }

     jnw = Ny-1; if (y_direction < 0) jnw = 0;
     for (i = 0; i < Nx; i++) {
       inw = i; if (x_direction < 0) inw = Nx-1-i; ij = inw*Ny+jnw;
       y2[i] = CDy - Az*(array[ij]-z_min);
     }
     if( (w_present_eagle == w_first_eagle) && (p_control == 1) )
       for (i = 0; i < Nx; i++) fprintf(file_id_eagle, "%d\n", y2[i]);

     for (j = Ny-1; j > 0; j--) {
       jnw = j-1; if (y_direction < 0) jnw = Ny-1-jnw;
       base = CDy + (y_width*(Ny-j))/(Ny-1);
       for (i = 0; i < Nx; i++) {
         y1[i] = y2[i]; inw = i; if (x_direction < 0) inw = Nx-1-i;
	 ij = inw*Ny+jnw; y2[i] = base - Az *(array[ij]-z_min);
       }
       if( (w_present_eagle == w_first_eagle) && (p_control == 1) )
         for (i = 0; i < Nx; i++) fprintf(file_id_eagle, "%d\n", y2[i]);

       for (i = 0; i < Nx-1; i++) {
         yy[0] = y2[i]; yy[1] = y2[i+1]; yy[2] = y1[i+1]; yy[3] = y1[i];
         xx[0] = x0 + (x_width*i)/(Nx-1)     + (x_diff*(j-1))/(Ny-1);
         xx[1] = x0 + (x_width*(i+1))/(Nx-1) + (x_diff*(j-1))/(Ny-1);
         xx[2] = x0 + (x_width*(i+1))/(Nx-1) + (x_diff*j)/(Ny-1);
         xx[3] = x0 + (x_width*i)/(Nx-1)     + (x_diff*j)/(Ny-1);
	 set_foreground(blue);  x_polygon(4, xx, yy, fpp);
         set_foreground(black);
	 if ( (i % x_intv) == 0 )        x_line(xx[3], yy[3], xx[0], yy[0]);
	 if ( ((Ny-1-j) % y_intv) == 0 ) x_line(xx[2], yy[2], xx[3], yy[3]);
	 if ( i == Nx - 2 )              x_line(xx[1], yy[1], xx[2], yy[2]);
	 if ( j == 1 )                   x_line(xx[0], yy[0], xx[1], yy[1]);
       }
     }
     set_foreground(forecolor); set_background(backcolor);
     p_control_eagle = p_control; free(y1); free(y2);
}
