/* LUp.c */
void LUPdecomp(A, B, C, D, E, N)
/*  LU-decomposition of a matrix for periodic boundary condition
       M[i,i]   = A[i]  (i=0,...,N-1)
       M[i,i+1] = B[i]  (i=0,...,N-3)
       M[i,i-1] = C[i]  (i=1,...,N-2) 
       M[i,N]   = D[i]  (i=0,...,N-2) 
       M[N,j]   = E[j]  (j=0,...,N-2)
       L[i,i]   = 1     (i=0,...,N-1)
       L[i,i-1] = -C[i] (i=1,...,N-2)
       L[N,j]   = -E[j] (j=0,...,N-2)
       U[i,i]   = A[i]  (i=0,...,N-1)
       U[i,i+1] = B[i]  (i=0,...,N-3)
       U[i,N]   = D[i]  (i=0,...,N-2)          */
    double A[], B[], C[], D[], E[]; int N;
{
    int j;
    E[0] = -E[0]/A[0]; A[N-1] = A[N-1]+D[0]*E[0];
    for (j=1; j<=N-2; j++) {
      C[j] = -C[j]/A[j-1]; A[j] = A[j]+C[j]*B[j-1]; D[j] = D[j]+C[j]*D[j-1];
      E[j] = -(E[j]+E[j-1]*B[j-1])/A[j] ; A[N-1] = A[N-1]+D[j]*E[j];
    }
}

void LUPsolve(A, B, C, D, E, x, N)
/*  Solve the system of linear equations Mx = b
    The matrix M should be LU-decomposed by LUPdecomp
    Right-hand vector b is stored in the array x    */
    double A[], B[], C[], D[], E[], x[]; int N;
{
    int j;
    x[N-1] = x[N-1]+E[0]*x[0];
    for (j=1; j<=N-2; j++)
    {  x[j] = x[j]+C[j]*x[j-1]; x[N-1] = x[N-1]+E[j]*x[j]; }
    x[N-1] = x[N-1]/A[N-1]; x[N-2] = (x[N-2]-D[N-2]*x[N-1])/A[N-2];
    for (j=N-3; j>=0; j--)
       x[j] = (x[j]-B[j]*x[j+1]-D[j]*x[N-1])/A[j];
}
